;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Achievement routines                                          ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; setAchievement - set an achievement as complete (if it wasn't already completed)
;   input:  C = achievement id #
;           B = bitmask
;           DE = offset in achievement data
;           HL => routine to use to show info box with new achievement info
;   output: none
;------------------------------------------------
setAchievement:
        push    hl                              ; save routine ptr
        ld      hl,(pAchFlags)
        add     hl,de                           ; HL => byte of achievement data to use
        pop     de                              ; DE => routine to use for info box
        ld      a,b
        and     (hl)                            ; check if achievement already completed
        ret     nz                              ; if so, we can leave
        ld      a,b
        or      (hl)                            ; set the bit
        ld      (hl),a                          ; save the new byte
        ex      de,hl
        jp      (hl)                            ; jump to routine for info box

;------------------------------------------------
; incAchievement - increase achievement progress (if it's not already completed)
;   input:  C = achievement id #
;           B = bitmask
;           DE = offset in achievement data    
;           HL = value to add
;   output: none
;------------------------------------------------
incAchievement:
        push    hl
        ld      hl,(pAchFlags)
        add     hl,de
        ld      a,b
        and     (hl)
        pop     hl
        ret     nz                              ; if already completed, leave
        ld      (tempX),bc                      ; save bitmask & id
        ld      (tempY),de                      ; save offset
        push    hl                              ; save increment value
        ld      a,c
        sub     ACH_HAS_VAR
        ld      b,a
        add     a,a
        add     a,b
        ld      l,a
        ld      h,0                             ; HL = variable / requirement offset
        push    hl
        ld      de,achieveReq
        add     hl,de
#ifdef TI83PLUS
        call    ldAHLind                        ; AHL = requirement value
        pop     de                              ; DE = offset
        ld      (temp24),hl
        ld      (temp24+2),a                    ; save requirement value
#endif
#ifdef TI84PLUSCE
        ld      hl,(hl)
        pop     de
        ld      (temp24),hl
#endif
        ld      hl,(pAchVars)
        add     hl,de                           ; HL => achievement variable
        pop     de
        push    hl
#ifdef TI83PLUS
        call    ldAHLind                        ; AHL = achievement variable
        add     hl,de
        adc     a,0                             ; AHL = updated value
        ex      de,hl
        ld      b,a                             ; BDE = updated value
        ld      hl,(temp24)
        ld      a,(temp24+2)                    ; AHL = requirement value
        call    cpAHLBDE
#endif
#ifdef TI84PLUSCE
        ld      hl,(hl)
        add     hl,de
        ex      de,hl
        ld      hl,(temp24)
        call    _cphlde
#endif
        pop     hl
        jr      c,incAchComplete
        jr      z,incAchComplete
#ifdef TI83PLUS
        jp      ldBDEtoHL
#endif
#ifdef TI84PLUSCE
        ld      (hl),de
        ret
#endif
incAchComplete:
#ifdef TI83PLUS
        ld      de,(temp24)
        ld      bc,(temp24+2-1)                 ; BDE = requirement value
        call    ldBDEtoHL
#endif
#ifdef TI84PLUSCE
        ld      de,(temp24)
        ld      (hl),de
#endif
        ld      bc,(tempX)                      ; B = bitmask, C = id
        ld      de,(tempY)                      ; DE = offset
        ld      hl,viewNewAchievementSaveScr
        jp      setAchievement

;------------------------------------------------
; viewNewAchievementSaveScr - show an info box with the newly completed achievement title & description (saves and restores gbuf, 83P only)
;   input:  C = id # for new achievement
;   output: none
;------------------------------------------------
viewNewAchievementSaveScr:
#ifdef TI83PLUS
        push    bc
        bcall(ab5_saveVideo)                    ; backup gbuf
        pop     bc
        call    viewNewAchievement
        bcall(ab5_restoreVideo)
        ret
#endif

;------------------------------------------------
; viewNewAchievement - show an info box with the newly completed achievement title & description
;   input:  C = id # for new achievement
;   output: none
;------------------------------------------------
viewNewAchievement:
        ld      l,c
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
#endif
#ifdef TI84PLUSCE
        ld      h,3
        mlt     hl
#endif
        ld      de,achieveInfo
        add     hl,de
        LDHLIND()                               ; HL => description info
#ifdef TI83PLUS
        ld      de,achBuffer+64
        push    de
        ld      bc,192
        ldir
        ld      hl,strNewAchievement            ; HL => title string for info box
        ld      de,achBuffer
        push    de
        ld      bc,64
        ldir
        pop     hl
        pop     de
        xor     a                               ; normal info box
        bcall(ab5_showInfoBox)
        ret
#endif
#ifdef TI84PLUSCE
        ex      de,hl                           ; put it in DE
        ld      hl,strNewAchievement            ; HL => title string for info box
        xor     a                               ; normal info box
        jp      showInfoBox
#endif

;------------------------------------------------
; viewAchievements - view the achievements screen
;   input:  none
;   output: none
;------------------------------------------------
viewAchievements:
        ; reset page # & line #
        xor     a
        ld      (menuNum),a
        ld      (menuItem),a
achieveScreen:
#ifdef TI83PLUS
        ; load the achievement screen, which is stored in line blocks
        ld      hl,achieveGbufData              ; HL => achievement screen background data
        ld      b,(hl)                          ; B = # data blocks to load
        inc     hl                              ; HL => first data block
        ld      de,gbuf                         ; DE => gbuf
achieveBgLoop:
        push    bc
        ld      b,(hl)                          ; B = # lines to repeat this block
        inc     hl                              ; HL => 12 bytes of line data
achieveBgBlock:
        push    bc
        push    hl
        ld      bc,12                           ; BC = 12 bytes per gbuf line
        ldir                                    ; load 12 bytes from (HL) to (DE) / data block to gbuf
        pop     hl
        pop     bc
        djnz    achieveBgBlock
        ld      bc,12
        add     hl,bc                           ; HL => next data block
        pop     bc
        djnz    achieveBgLoop
        ld      hl,strModeBack
        call    copy256Bytes
        bcall(ab5_showStrings)                  ; "[MODE]-BACK"
#endif
#ifdef TI84PLUSCE
        call    vbufClear
        RECT_POS(32,32)
        RECT_SIZE(256,176)
        ld      a,COLOUR_LIGHTBLUE
        call    drawWindow
        ld      ix,strModeBack
        call    printBlock
#endif
        ; write the achievement names
#ifdef TI83PLUS
        ld      hl,(menuNum)
        ld      h,0
        add     hl,hl
        ld      de,achieveCats+1
        add     hl,de
        LDHLIND()                               ; HL => strings to write
        call    copy256Bytes
        bcall(ab5_showStrings)
#endif
#ifdef TI84PLUSCE
        ld      hl,(menuNum)
        ld      h,3
        mlt     hl
        ld      de,achieveCats+1
        add     hl,de
        LDHLIND()
        call    printBlockHL
#endif
        ; draw the tick sprites for any completed achievements
#ifdef TI83PLUS
        ld      hl,sprAchieveTick
        call    copySprite                      ; copy tick sprite to ram
#endif
        ld      hl,(pAchFlags)
#ifdef TI83PLUS
        ld      de,(menuNum)
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ld      a,(menuNum)
        ld      de,0
        ld      e,a
#endif
        add     hl,de
        ld      c,(hl)                          ; C = byte of data for this achievement category
        ld      b,5                             ; B = 5 achievements to check
#ifdef TI83PLUS
        ld      l,11                            ; L = y coord to start at
achieveTickLoop:
        srl     c                               ; shift C right 1 bit to check if achievement is completed
        push    bc                              ; save counter & bit data
        jr      nc,afterAchieveTick             ; if bit was reset, achievement has not been completed, so skip drawing the tick
        push    hl                              ; save y coord
        ld      a,3                             ; A = x coord
        ld      de,sprBuffer                    ; DE => sprite
        bcall(ab5_putSprite)
        pop     hl                              ; restore y coord
afterAchieveTick:
        ld      a,6
        add     a,l
        ld      l,a                             ; L = next y coord
        pop     bc                              ; restore counter & bit data
        djnz    achieveTickLoop
#endif
#ifdef TI84PLUSCE
        ld      a,60
achieveTickLoop:
        srl     c
        push    bc
        jr      nc,afterAchieveTick
        ld      ix,strAchieveTick
        ld      (ix+1),a
        push    af
        call    printBlock
        pop     af
afterAchieveTick:
        add     a,15
        pop     bc
        djnz    achieveTickLoop
#endif
        ; draw the cursor
#ifdef TI83PLUS
        ld      hl,sprAchieveCursor
        call    copySprite                      ; copy cursor sprite to ram
        ld      hl,(menuItem)
        ld      h,6
        bcall(_htimesl)
        ld      a,11
        add     a,l
        ld      l,a                             ; L = y coord
        ld      a,3                             ; A = x coord
        ld      de,sprBuffer                    ; DE => sprite
        bcall(ab5_putSprite)
#endif
#ifdef TI84PLUSCE
        ld      a,(frame)
        inc     a
        ld      (frame),a
        call    updateCursorFlash
        ld      hl,(menuItem)
        ld      h,15
        mlt     hl
        ld      a,60
        add     a,l
        LDHLA()
        ld      h,160
        mlt     hl
        add     hl,hl
        ld      de,(pVbuf)
        add     hl,de
        ld      de,38
        add     hl,de
        ld      de,320-7
        ld      c,11
acOuterLoop:
        ld      b,7
acInnerLoop:
        ld      (hl),$FF
        inc     hl
        djnz    acInnerLoop
        add     hl,de
        dec     c
        jr      nz,acOuterLoop
#endif
#ifdef TI83PLUS
        ; invert the gbuf
        bcall(ab5_invertGbuf)
#endif
        ; write the description
#ifdef TI83PLUS
        ld      hl,(menuNum)
        ld      h,10
        bcall(_htimesl)
        ex      de,hl                           ; DE = category offset
        ld      hl,(menuItem)
        ld      h,0
        add     hl,hl
        add     hl,de                           ; HL = offset
        ld      de,achieveDescs
        add     hl,de
        LDHLIND()                               ; HL => string data
        call    copy256Bytes
        bcall(ab5_showStrings)
#endif
#ifdef TI84PLUSCE
        ld      hl,(menuNum)
        ld      h,15
        mlt     hl
        ld      de,(menuItem)
        ld      d,3
        mlt     de
        add     hl,de
        ld      de,achieveDescs
        add     hl,de
        LDHLIND()
        call    printBlockHL
#endif
        ; write the progress (if required for this achievement)
#ifdef TI83PLUS
        ld      de,54*256+8
        ld      (pencol),de                     ; set cursor position
        ld      hl,(menuNum)
        ld      h,5
        bcall(_htimesl)
        ld      a,l
        ld      hl,menuItem
        add     a,(hl)                          ; A = achievement ID
        cp      ACH_HAS_VAR
        jr      nc,showAchieveCounter
        cp      ACH_EP3PATHS_ID
        jr      nz,afterAchieveProgress
        ; calc number of routes taken in Ep3
        ld      hl,(pAchVars)
        ld      de,ACH_COUNTER_SIZE
        add     hl,de
        ld      c,(hl)
        ld      b,7
        xor     a
calcRoutesLoop:
        srl     c
        adc     a,0
        djnz    calcRoutesLoop
        LDHLA() \ xor a                         ; AHL = A
        push    de                              ; DE = ACH_COUNTER_SIZE from earlier
        jr      showAchieveCounterDisp
showAchieveCounter:
        sub     ACH_HAS_VAR
        ld      b,a
        add     a,a
        add     a,b
        ld      e,a
        ld      d,0                             ; DE = offset
        push    de                              ; save offset
        ld      hl,(pAchVars)
        add     hl,de                           ; HL => achievement variable
        call    ldAHLind
showAchieveCounterDisp:
        call    showAHL
        ld      hl,strAchProgress
        call    copy256Bytes
        bcall(ab5_vPutsNoCursor)
        pop     de                              ; restore offset
        ld      hl,achieveReq
        add     hl,de
        call    ldAHLind
        call    showAHL
#endif
#ifdef TI84PLUSCE
        ld      hl,(menuNum)
        ld      h,5
        mlt     hl
        ld      a,l
        ld      hl,menuItem
        add     a,(hl)
        cp      ACH_HAS_VAR
        jr      nc,showAchieveCounter
        cp      ACH_EP3PATHS_ID
        jr      nz,afterAchieveProgress
        ; calc number of routes taken in Ep3
        ld      hl,(pAchVars)
        ld      de,ACH_COUNTER_SIZE
        add     hl,de
        ld      c,(hl)
        ld      b,7
        xor     a
calcRoutesLoop:
        srl     c
        adc     a,0
        djnz    calcRoutesLoop
        LDHLA()
        push    de                              ; DE = ACH_COUNTER_SIZE from earlier
        jr      showAchieveCounterDisp
showAchieveCounter:
        sub     ACH_HAS_VAR
        ld      e,a
        ld      d,3
        mlt     de
        push    de
        ld      hl,(pAchVars)
        add     hl,de
        ld      hl,(hl)
showAchieveCounterDisp:
        ld      de,48
        ld      c,180
        call    showHLLeft
        ld      ix,strAchProgress
        call    printString
        pop     bc
        ld      hl,achieveReq
        add     hl,bc
        ld      hl,(hl)
        ld      c,180
        call    showHLLeft
#endif
afterAchieveProgress:
#ifdef TI83PLUS
        ; copy gbuf to lcd
        bcall(_grbufcpy)
#endif
#ifdef TI84PLUSCE
        call    vramFlip
#endif
achieveKey:
#ifdef TI83PLUS
        ; wait for a key and process it
        bcall(ab5_waitKey)
#endif
#ifdef TI84PLUSCE
        call    _getcsc
#endif
        cp      GK_MODE                         ; [MODE]?
        ret     z
        ld      hl,menuItem                     ; used in achieveNextLine & achievePrevLine
        cp      GK_UP                           ; [UP]?
        jr      z,achievePrevLine
        cp      GK_DOWN                         ; [DOWN]?
        jr      z,achieveNextLine
        ld      hl,achieveCats                  ; used in achieveNextCat & achievePrevCat
        ld      de,menuNum                      ; ditto
        cp      GK_LEFT                         ; [LEFT]?
        jr      z,achievePrevCat
        cp      GK_RIGHT                        ; [RIGHT]?
        jr      z,achieveNextCat
#ifdef TI83PLUS
        jr      achieveKey
#endif
#ifdef TI84PLUSCE
        jp      achieveScreen
#endif

;------------------------------------------------
; achieveNextLine - move to the next line
;   input:  HL => menuItem
;   output: none
;------------------------------------------------
achieveNextLine:
        ld      a,(hl)
        inc     a
        cp      5
        jr      nz,saveAchieveLine
        xor     a
        jr      saveAchieveLine

;------------------------------------------------
; achievePrevLine - move to the previous line
;   input:  HL => menuItem
;   output: none
;------------------------------------------------
achievePrevLine:
        ld      a,(hl)
        dec     a
        bit     7,a
        jr      z,saveAchieveLine
        ld      a,4
        ; fall through to saveAchieveLine

;------------------------------------------------
; saveAchieveLine - save item # and jump back to redraw the screen
;   input:  HL => menuItem
;   output: none
;------------------------------------------------
saveAchieveLine:
        ld      (hl),a
        jp      achieveScreen

;------------------------------------------------
; achieveNextCat - move to the next category
;   input:  HL => achieveCats
;           DE => menuNum
;   output: none
;------------------------------------------------
achieveNextCat:
        ld      a,(de)
        inc     a
        cp      (hl)                            ; have we passed the last category?
        jr      nz,saveAchieveCat
        xor     a
        jr      saveAchieveCat

;------------------------------------------------
; achievePrevCat - move to the previous category
;   input:  HL => achieveCats
;           DE => menuNum
;   output: none    
;------------------------------------------------
achievePrevCat:
        ld      a,(de)
        dec     a
        bit     7,a                             ; did we go to -1?
        jr      z,saveAchieveCat                ; if we didn't, we're done
        ld      a,(hl)
        dec     a
        ; fall through to saveAchieveCat

;------------------------------------------------
; saveAchieveCat - save category # and jump back to redraw the screen
;   input:  DE => menuNum
;   output: none
;------------------------------------------------
saveAchieveCat:
        ld      (de),a
        jp      achieveScreen

.end
