;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Cut scene routines                                            ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; doCutScene - show a cut scene
;   input:  C = offset to app page with cut scene data (non-CE only)
;           HL => compressed cut scene data
;   output: none
;------------------------------------------------
doCutScene:
        push    bc
        push    hl
        call    iniLevelLoad                    ; use the same routine as level loading
        ; copy gbuf to temp file (1536 bytes in to allow room for cut scene data)
        ld      hl,(tempFile)
        ld      de,1536
        add     hl,de
        ex      de,hl
        ld      hl,gbuf
        ld      bc,768
        ldir                                    ; save gbuf
        pop     hl
        pop     bc
        ld      b,0
        ld      de,(tempFile)
        call    ram_huffExtr                    ; call routine in ram, which will swap to correct rom page, decompress the level data to ram, and revert to this rom page
        ; restore gbuf from temp file
        ld      hl,(tempFile)
        ld      de,1536
        add     hl,de
        ld      de,gbuf
        ld      bc,768
        ldir
        call    splashAnim                      ; animate the background
        ld      b,200
        call    shortDelay                      ; show background for a short period before text starts displaying
        ret     c                               ; if [ALPHA] was pressed during delay, leave
        ld      hl,(tempFile)                   ; HL => uncompressed cut screen data
        ld      bc,768
        add     hl,bc                           ; HL => text data
        ld      b,(hl)                          ; B = # text panels
        inc     hl
cutSceneLoop:
        push    bc                              ; save counter
        push    hl                              ; save ptr to text panel
        call    cutSceneLoadImg
        pop     hl                              ; HL => text panel data
        call    cutSceneTextBox                 ; display the text box
        call    displayPageNoClear              ; display text in the box using the gbuf background
        push    hl
        ld      bc,500
        call    longDelay
        pop     hl
        pop     bc
        jr      c,cutSceneLoadImg               ; if [ALPHA] was pressed during delay, skip out
        djnz    cutSceneLoop
        jr      cutSceneLoadImg                 ; reload background image and ret
cutSceneSkip:
        pop     bc                              ; clear stack
        ; fall through to cutSceneLoadImg and ret

;------------------------------------------------
; cutSceneLoadImg - load the cut scene background image
;   input:  (tempFile) => image data
;   output: none
;------------------------------------------------
cutSceneLoadImg:
        ld      hl,(tempFile)                   ; HL => image data
        ld      de,gbuf
        ld      bc,768
        ldir                                    ; load image to gbuf
        ret

;------------------------------------------------
; cutSceneTextBox - draw box for cut scene text to be displayed in
;   input:  HL => {top y pos, bottom y pos} (byte line values)
;   output: HL => data after text box position
;------------------------------------------------
cutSceneTextBox:
        push    hl                              ; save ptr to top line pos
        ld      l,(hl)
        ld      h,12
        bcall(_htimesl)
        ld      de,gbuf
        add     hl,de                           ; HL => line on gbuf to draw top border
        call    textBoxBorderLine               ; draw top border
        pop     de
        inc     de
        ld      a,(de)
        ld      b,a
        push    de
cutSceneTextBoxLines:
        push    bc
        call    textBoxTextLine
        pop     bc
        djnz    cutSceneTextBoxLines
        call    textBoxBorderLine               ; draw bottom border
        pop     hl
        inc     hl
        ret

;------------------------------------------------
; textBoxBorderLine - draw a solid horizontal line
;   input:  HL => row on gbuf to draw to
;   output: HL => next row on gbuf
;------------------------------------------------
textBoxBorderLine:
        ld      b,12
tbblLoop:
        ld      (hl),$FF
        inc     hl
        djnz    tbblLoop
        ret

;------------------------------------------------
; textBoxTextLine - draw a white horizontal line with black pixels at left and right
;   input:  HL => row on gbuf to draw to
;   output: HL => next row on gbuf
;------------------------------------------------
textBoxTextLine:
        ld      (hl),%10000000
        inc     hl
        ld      b,10
tbtlLoop:
        ld      (hl),$00
        inc     hl
        djnz    tbtlLoop
        ld      (hl),%00000001
        inc     hl
        ret

.end
