;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Main game loop                                                ;
;                                                               ;
;===============================================================;

mainGameLoop:
#ifdef TI84PLUSCE
        FPS_FRAME()
#endif
#ifdef ALL_ITEMS
        ld      a,%00111111
        ld      (weaponTable),a
#endif
        ; do a couple of initial frame tasks
#ifdef TI83PLUS
        xor     a
        ld      (timer),a                       ; reset internal timer (used to make sure game doesn't run too fast)
        ld      a,APD_MAIN
        ld      (apdMain),a                     ; ensure apd main timer is reset to avoid apd shutdowns
#endif
#ifdef TI84PLUSCE
        call    keyScan
#endif
        ld      hl,frame
        inc     (hl)                            ; inc frame counter
        ld      a,(hl)
        and     %00000011                       ; every 4th frame?
        call    z,trgFrame4
        ld      hl,miscFlags
        res     p1Blast,(hl)                    ; clear flag to draw weapon blast effect in drawBullets
        res     p2Blast,(hl)

        ld      ix,p1x
        call    checkPlayerDead                 ; check if player 1 is dead, and process if required
        ld      ix,p2x
        call    checkPlayerDead                 ; and for player 2
        ld      a,(p1p)
        ld      hl,p2p
        add     a,(hl)
        bcallz(ab5_pdeadGameOver)               ; if no players active, game over

        ld      hl,p1i
        call    checkPlayerInvincible
        ld      hl,p2i
        call    checkPlayerInvincible

        ; check if level is finished
        ld      hl,levelFlags
        bit     LFLAG_CANFINISH,(hl)
        jr      z,levelNotFinished              ; if flag not set, can't finish the level yet
        ld      hl,p1x
        call    checkLevelFinished
        jr      nc,levelNotFinished
        ld      hl,p2x
        call    checkLevelFinished
        bcallc(ab5_levelFinished)
        
levelNotFinished:
        ld      hl,levelFlags
        bit     LFLAG_COUNTDOWNON,(hl)          ; is countdown running?
        jr      z,afterDestruct

        ; if execution has reached here, the countdown sequence is running
        ; maybe make a random explosion
        ld      a,(time)                        ; A = time left on countdown
        srl     a                               ; A = time / 2
        add     a,20
        ld      b,a
        call    getRandom                       ; A = {0 .. (B-1)}
        or      a                               ; check if zero
        call    z,randomExplosion               ; if so, make a new random explosion on screen

        ; decrement and check the countdown timer (unless the cheat is active)
        ld      a,(frame)
        and     %00011111
        jr      nz,afterDestruct                ; only adjust countdown timer every 32nd frame
        ld      hl,e2cheatFlags
        bit     noCountdownCheat,(hl)
        jr      nz,afterDestruct
        ld      hl,time
        dec     (hl)                            ; decrement timer
        bit     7,(hl)                          ; check if it's gone below 0
        bcallnz(ab5_timerZero)                  ; if so, the level explodes and the game is over

afterDestruct:
        ; update alien holes, animations, aliens, bullets & boss if one is active
        call    updateAlienHoles                ; update alien holes
        ld      a,(frame)
        bit     0,a
        call    z,updateAnims                   ; update animations (only every 2nd frame)
        call    updateObjects                   ; update objects
        call    moveEnemies                     ; move enemies
        call    doEbullet                       ; update enemy bullet
        call    moveBullets                     ; move player bullets
        ld      a,(state)
        or      a
        call    nz,moveBoss                     ; move boss if one is active

        ; check if player is standing on normal floor, or low bare/exposed floor
        ; this will determine the player/wall collision checking if the player tries to move
        ; check player wall collision tiles
        ld      a,(p1p)
        or      a
        ld      ix,p1x
        call    nz,calcPlayerCt
        ld      a,(p2p)
        or      a
        ld      ix,p2x
        call    nz,calcPlayerCt

        ; handle key presses
        call    checkKeys

        ; was a teleport encountered?
        ld      hl,miscFlags
        bit     teleportCmd,(hl)
        jp      nz,teleportPlayer

        ; calculate average position of active players relative to screen
#ifdef TI83PLUS
        ld      hl,(p1scry)
        ld      (ascry),hl                      ; set averages equal to player 1
#endif
#ifdef TI84PLUSCE
        ld      a,(p1scry)
        ld      (ascry),a
        ld      a,(p1scrx)
        ld      (ascrx),a
#endif
        ld      a,(coop)
        or      a
        jr      z,checkScreenScroll             ; if a 1 player game, no average calculating to do
        ld      a,(p2p)
        or      a
        jr      z,checkScreenScroll             ; if player 2 not active, just go with player 1 position
#ifdef TI83PLUS
        ld      hl,(p2scry)
        ld      (ascry),hl                      ; set averages equal to player 2
#endif
#ifdef TI84PLUSCE
        ld      a,(p2scry)
        ld      (ascry),a
        ld      a,(p2scrx)
        ld      (ascrx),a
#endif
        ld      a,(p1p)
        or      a
        jr      z,checkScreenScroll             ; if player 1 not active, use player 2 position
        ; otherwise, get an average with player 2 values
        ld      hl,(p1y)
        ld      de,(p2y)
        call    average                         ; HL = average of both player y coords
        ld      de,(screeny)
        or      a                               ; clear carry
        sbc     hl,de                           ; L = centered y coord between both players
        push    hl                              ; save it
        ld      hl,(p1x)
        ld      de,(p2x)
        call    average
        ld      de,(screenx)
        or      a
        sbc     hl,de
        ld      h,l                             ; H = centered x coord
        pop     de
        ld      l,e                             ; L = centered y coord
        ld      (ascry),hl                      ; save them

checkScreenScroll:
        ; check for required screen scrolling based on updated player positions
        ; check if screen needs to be scrolled horizontally
        ld      a,(ascrx)
        push    af
        cp      SCROLL_RIGHT
        call    c,scrollRight                   ; only try to scroll right if players have moved too far left on screen
        pop     af
        cp      SCROLL_LEFT
        call    nc,scrollLeft                   ; only try to scroll left if players have moved too far right on screen
#ifdef TI83PLUS
        ; check if screen needs to be scrolled vertically (parameters vary depending on whether fighting a boss or not)
        ld      a,(state)
        or      a
        ld      a,SCROLL_DOWN                   ; first cp value in the case of not fighting a boss
        jr      z,loadVScrollCheck
        ld      b,a
        ld      a,(p1p)
        ld      hl,p2p
        add     a,(hl)
        cp      2
        ld      a,b
        jr      z,loadVScrollCheck              ; if 2 players, don't do the alternate scroll thing
        ld      a,SCROLL_DOWN_BOSS              ; first cp value in the case of fighting a boss
#endif
#ifdef TI84PLUSCE
        ld      a,SCROLL_DOWN
#endif
loadVScrollCheck:
        ld      (vsc1),a                        ; load first cp value
        add     a,SCROLL_V_ADD
        ld      (vsc2),a                        ; load second cp value
        ld      a,(ascry)
        push    af
        ld      hl,vsc1                         ; HL => vsc1
        cp      (hl)
        call    c,scrollDown                    ; only try to scroll down if players have moved to far up on screen
        pop     af
        ld      hl,vsc2
        cp      (hl)
        call    nc,scrollUp                     ; only try to scroll up if players have moved too far down on screen

        ; check collisions
        ld      a,(state)
        or      a                               ; are we fighting a boss?
        jr      z,afterBossCollisions           ; if not, skip boss collision checking
        call    checkBulletBossCollisions       ; check bullet/boss collisions
        ld      a,(p1p)
        or      a
        call    nz,checkP1BossCollisions        ; check player/boss collisions
        ld      a,(p2p)
        or      a
        call    nz,checkP2BossCollisions
afterBossCollisions:
        call    checkBulletEnemyCollisions      ; check bullet/enemy collisions
        ld      a,(p1p)
        or      a
        call    nz,checkP1EnemyCollisions       ; check player/enemy & enemy bullet collisions
        ld      a,(p2p)
        or      a
        call    nz,checkP2EnemyCollisions       ; check player/enemy & enemy bullet collisions

        ld      hl,e1cheatFlags
        bit     invincibleCheat,(hl)
        jr      z,afterInvincibleCheat
        ld      a,MAX_HEALTH
        ld      (p1h),a                         ; keep player health at full
        ld      (p2h),a
        xor     a
        ld      (p1ht),a                        ; clear player hurt flag
        ld      (p2ht),a
afterInvincibleCheat:
        ; check if either player was hurt, and if so, set showHealth counter
        ld      a,(p1p)
        ld      hl,p1ht
        and     (hl)
        jr      z,afterSetShowHealthP1
        ld      a,90
        ld      (p1sh),a                        ; set counter to show health meter for the next A frames
        xor     a
        ld      (aflagBossCore),a               ; clear this flag in case it's a boss fight, which denotes that player got killed
afterSetShowHealthP1:
        ld      a,(p2p)
        ld      hl,p2ht
        and     (hl)
        jr      z,afterSetShowHealthP2
        ld      a,90
        ld      (p2sh),a
        xor     a
        ld      (aflagBossCore),a
afterSetShowHealthP2:

        ; check tile at centre of both players
        ld      ix,p1x
        ld      a,(ix+PP)
        or      a
        call    nz,checkCentreTile
        ld      ix,p2x
        ld      a,(ix+PP)
        or      a
        call    nz,checkCentreTile

        ; save gbuf, draw sprites, copy gbuf to lcd, restore gbuf
#ifdef TI83PLUS
        call    saveVideo                       ; save copy of gbuf
#endif

#ifdef TI83PLUS
        ld      a,(state)
        or      a
        call    nz,drawBoss                     ; draw boss if in boss mode
        call    drawObjects                     ; draw objects to gbuf
        call    drawAnims                       ; draw animations to gbuf
        call    drawBullets                     ; draw player bullets to gbuf
        call    drawEbullet                     ; draw enemy bullet to gbuf
        call    drawEnemies                     ; draw enemies to gbuf
        call    drawPlayers                     ; draw players to gbuf
        ld      a,(levelFlags)
        bit     LFLAG_DARKLEVEL,a
        call    nz,darkMask                     ; if a dark level, draw the mask overlay

        ld      hl,levelFlags
        bit     LFLAG_COUNTDOWNON,(hl)          ; is countdown running?
        jr      z,afterShakeTimer
        call    countdownShake
        call    showTimer
afterShakeTimer:

        ; show health meters if required
        ld      hl,P1HMETER
        ld      bc,p1sh
        ld      de,p1h
        call    checkShowHealth
        ld      hl,P2HMETER
        ld      bc,p2sh
        ld      de,p2h
        call    checkShowHealth
#endif

#ifdef TI83PLUS
        ; check for inverted lcd cheat
        ld      hl,e2cheatFlags
        bit     invertedLcdCheat,(hl)
        call    nz,gbufInvert
#endif

#ifdef TI83PLUS
        call    gbufCopy                        ; update lcd
        call    restoreVideo                    ; restore gbuf from before sprites were drawn
#endif

#ifdef TI84PLUSCE
        call    updateCountdownPalette          ; continue palette shift if countdown was recently started
        call    drawTiles
        call    drawObjects
        call    drawEnemies
        ld      a,(state)
        or      a
        call    nz,drawBoss
        call    drawBullets
        call    drawEBullet
        call    drawAnims
        call    drawPlayers
        ld      hl,levelFlags
        bit     LFLAG_COUNTDOWNON,(hl)
        jr      z,afterShakeTimer
        call    countdownShake
        call    showTimer
afterShakeTimer:
        call    showHud
#ifdef FPS_TIMER_ON
        ld      hl,(fpsDisp)
        ld      de,0
        ld      c,24
        call    showHL
#endif
        call    vramFlip
#endif

        ei                                      ; make sure interrupts are enabled
#ifdef TI83PLUS
        set     apdable,(iy+apdflags)           ; ensure apd is enabled
        set     apdrunning,(iy+apdflags)
        ld      hl,timer                        ; HL => internal timer
mainLoopWait:
        ld      a,(hl)
        neg                                     ; timer counts backwards
        cp      FRAME_TIMER                     ; has this frame taken long enough?
        jr      nc,afterMainLoopWait            ; if so, we're done
        halt                                    ; otherwise, wait a little
        jr      mainLoopWait

afterMainLoopWait:
#endif
        ; check if player(s) died this frame
        ld      hl,p1dd
        ld      de,p1h
        ld      bc,p1l
        call    checkPlayerDied
        ld      hl,p2dd
        ld      de,p2h
        ld      bc,p2l
        call    checkPlayerDied
        jp      mainGameLoop

.end
